@@ -7,6 +7,7 @@ from typing import Tuple, Dict, Any, List
 
 import aiofiles
 from gcp_pilot.datastore import Document, DoesNotExist
+from gcp_pilot.exceptions import ValidationError
 from sanic.request import Request, RequestParameters, File
 from sanic.response import json, HTTPResponse
 from sanic.views import HTTPMethodView
@@ -54,9 +55,14 @@ class ViewBase(HTTPMethodView):
 
 
 class ListView(ViewBase, abc.ABC):
+    search_field: str = None
+
     async def get(self, request: Request) -> HTTPResponse:
         query_args, page, page_size = self._parse_query_args(request=request)
-        items = await self.perform_get(query_filters=query_args)
+        try:
+            items = await self.perform_get(query_filters=query_args)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
 
         items_in_page = self._paginate(items=items, page=page, page_size=page_size)
 
@@ -72,8 +78,11 @@ class ListView(ViewBase, abc.ABC):
         return json(response, 200 if any(items_in_page) else 404)
 
     def _parse_query_args(self, request: Request) -> Tuple[Dict[str, Any], int, int]:
-        query_args = {}
+        query_args = dict()
         for key, value in request.query_args:
+            if key == 'q':
+                key = f'{self.search_field}__startswith'
+
             if key not in query_args:
                 query_args[key] = value
             elif isinstance(query_args[key], list):
@@ -111,7 +120,12 @@ class ListView(ViewBase, abc.ABC):
     async def post(self, request: Request) -> HTTPResponse:
         data = request.json
         self.validate(data=data, partial=False)
-        obj = await self.perform_create(data=data)
+
+        try:
+            obj = await self.perform_create(data=data)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
         data = obj.serialize()
         return json(data, 201)
 
@@ -133,14 +147,16 @@ class DetailView(ViewBase, abc.ABC):
         pk_field_type = self.model.Meta.fields[self.model.Meta.pk_field]
         try:
             return pk_field_type(pk)
-        except ValueError:
-            raise exceptions.ValidationError(f"{pk_field_name} field must be {pk_field_type.__name__}")
+        except ValueError as e:
+            raise exceptions.ValidationError(f"{pk_field_name} field must be {pk_field_type.__name__}") from e
 
     async def get(self, request: Request, pk: str) -> HTTPResponse:
         try:
-            obj = self.perform_get(pk=self._parse_pk(pk=pk), query_filters={})
+            obj = await self.perform_get(pk=self._parse_pk(pk=pk), query_filters={})
         except DoesNotExist as e:
             raise exceptions.NotFoundError() from e
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
 
         data = obj.serialize()
         return json(data, 200)
@@ -151,7 +167,12 @@ class DetailView(ViewBase, abc.ABC):
     async def put(self, request: Request, pk: str) -> HTTPResponse:
         payload = request.json
         self.validate(data=payload, partial=True)
-        obj = await self.perform_create(data=payload)
+
+        try:
+            obj = await self.perform_create(data=payload)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
         data = obj.serialize()
         return json(data, 200)
 
@@ -165,7 +186,11 @@ class DetailView(ViewBase, abc.ABC):
         else:
             payload = request.json
             self.validate(data=payload, partial=True)
-        obj = await self.perform_update(pk=self._parse_pk(pk=pk), data=payload, files=request.files)
+
+        try:
+            obj = await self.perform_update(pk=self._parse_pk(pk=pk), data=payload, files=request.files)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
 
         data = obj.serialize()
         return json(data, 200)
@@ -212,7 +237,11 @@ class ActionView(ViewBase):
         except DoesNotExist as e:
             raise exceptions.NotFoundError() from e
 
-        data, status = await self.perform_get(request=request, obj=obj)
+        try:
+            data, status = await self.perform_get(request=request, obj=obj)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
         return json(data, status)
 
     @abc.abstractmethod
@@ -225,13 +254,34 @@ class ActionView(ViewBase):
         except DoesNotExist as e:
             raise exceptions.NotFoundError() from e
 
-        data, status = await self.perform_post(request=request, obj=obj)
+        try:
+            data, status = await self.perform_post(request=request, obj=obj)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
         return json(data, status)
 
     @abc.abstractmethod
     async def perform_post(self, request: Request, obj: Document) -> ResponseType:
         raise NotImplementedError()
 
+    async def put(self, request: Request, pk: str) -> HTTPResponse:
+        try:
+            obj = self.get_model(pk=pk)
+        except DoesNotExist as e:
+            raise exceptions.NotFoundError() from e
+
+        try:
+            data, status = await self.perform_put(request=request, obj=obj)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
+        return json(data, status)
+
+    @abc.abstractmethod
+    async def perform_put(self, request: Request, obj: Document) -> ResponseType:
+        raise NotImplementedError()
+
     async def delete(self, request: Request, pk: str) -> HTTPResponse:
         try:
             obj = self.get_model(pk=pk)
